@@ -1,7 +1,7 @@
 from pathlib import Path
 
 
-from pathy import PureGCSPath
+from pathy import PurePathy
 from pathy.file import BucketClientFS
 
 
@@ -9,5 +9,5 @@ def test_client_create_bucket(temp_folder: Path):
     bucket_target = temp_folder / "foo"
     assert bucket_target.exists() is False
     cl = BucketClientFS(temp_folder)
-    cl.create_bucket(PureGCSPath("gs://foo/"))
+    cl.create_bucket(PurePathy("gs://foo/"))
     assert bucket_target.exists() is True
